#!/usr/bin/perl -w

use Getopt::Long;

my %STATUS = ( 'OK' => 0, 'WARNING' => 1, 'CRITICAL' => 2, 'UNKNOWN' => 3);
my $version = "0.1";		# Version number
my $display_version = 0;	# Display version information
my $help = 0;
my $warning_level = "";
my $critical_level = "";
my $threshold_type = "amount";
my $verbose = 0;
my $method = "top";			# Either top or swap

sub version {
	print <<EOV;
check_swap version $version

Check swap swap space on Solaris.
EOV
}

sub help {
	print <<EOH;
Check swap space on local machine.

Usage: check_swap [-v] -w <percent_free>% -c <percent_free>%
       check_swap [-v] -w <amount_free> -c <amount_free>

Options:
 -h, --help
    Print detailed help screen
 -V, --version
    Print version information
 -w, --warning=INTEGER[kMGT]
    Exit with WARNING status if less than INTEGER amount of swap space is free
 -w, --warning=PERCENT%%
    Exit with WARNING status if less than PERCENT of swap space is free
 -c, --critical=INTEGER[kMGT]
    Exit with CRITICAL status if less than INTEGER amount of swap space is free
 -c, --critical=PERCENT%%
    Exit with CRITCAL status if less than PERCENT of swap space is free
 -m, --method
    "top" - uses top to determine swap usage (default)
		top must be installed separately, but top method does not include RAM in
		calculation
	"swap" - uses swap to determine swap usage
		swap is provided with Solaris, but includes physical RAM in it calculation
 -v, --verbose
    Increase the level of verbosity (maximum value of 3)
EOH

}

sub bytes2uom {
	my $bytes = shift;
	my $uom = shift;

	if( $uom eq "k") {
		return sprintf( "%lukB", $bytes / 1024);
	}
	elsif( $uom eq "M") {
		return sprintf( "%luMB", $bytes / 1024**2);
	}
	elsif( $uom eq "G") {
		return sprintf( "%luGB", $bytes / 1024**3);
	}
	elsif( $uom eq "T") {
		return sprintf( "%luTB", $bytes / 1024**4);
	}
	else {
		return "Unknown UOM";
	}
}

# Get the command line arguments
GetOptions(
	'h'		=> \$help,				'help'			=> \$help,
	'V'		=> \$display_version,	'version'		=> \$display_version,
									'verbose+'		=> \$verbose,
	'w:s'	=> \$warning_level,		'warning:s'		=> \$warning_level,
	'c:s'	=> \$critical_level,	'critical:s'	=> \$critical_level,
	'm:s'	=> \$method,			'method:s'		=> \$method);

# Validate the arguments
if( $display_version) {
	version();
	exit( $STATUS{ 'OK'});
}

if( $help) {
	help();
	exit( $STATUS{ 'OK'});
}

if( $verbose > 3) {
	print STDERR "WARNING: Excessive verbosity requsted; using verbosity of 3.\n";
}

if( $method !~ /^top$|^swap$/) {
	print "UNKNOWN: Illegal method $method specified for determining swap usage.\n";
	exit( $STATUS{ 'UNKNOWN'});
}

if( $warning_level eq "" || $critical_level eq "") {
	print "UNKNOWN: You must specify both a warning and a cricital threshold.\n";
	exit( $STATUS{ 'UNKNOWN'});
}

if( $warning_level =~ /\%$/) {
	$warning_level =~ s/\%$//;
	$threshold_type = "percent";
	if( $critical_level !~ /\%$/) {
		print "UNKNOWN: Both warning and cricital thresholds must be of the same type (amount or %).\n";
		exit( $STATUS{ 'UNKNOWN'});
		
	}
	$critical_level =~ s/\%$//;
}

if( $threshold_type eq "amount" && $critical_level =~ /\%$/) {
	print "UNKNOWN: Both warning and cricital thresholds must be of the same type (amount or %).\n";
	exit( $STATUS{ 'UNKNOWN'});
}

my( $warning_uom, $critical_uom);

if( $warning_level =~ /^([0-9\.]+)(([kMGT])?)$/) {
	( $warning_level, $warning_uom) = ( $1, $2);
	$warning_level *= 1024 if( $warning_uom eq "k");
	$warning_level *= 1024**2 if( $warning_uom eq "M");
	$warning_level *= 1024**3 if( $warning_uom eq "G");
	$warning_level *= 1024**4 if( $warning_uom eq "T");
}
else {
	print "UNKNOWN: Warning level must be a numerical value.\n";
	exit( $STATUS{ 'UNKNOWN'});
}

if( $critical_level =~ /^([0-9\.]+)(([kMGT])?)$/) {
	( $critical_level, $critical_uom) = ( $1, $2);
	$critical_level *= 1024 if( $critical_uom eq "k");
	$critical_level *= 1024**2 if( $critical_uom eq "M");
	$critical_level *= 1024**3 if( $critical_uom eq "G");
	$critical_level *= 1024**4 if( $critical_uom eq "T");
}
else {
	print "UNKNOWN: Critical level must be a numerical value.\n";
	exit( $STATUS{ 'UNKNOWN'});
}


if( $warning_level < $critical_level) {
	print "UNKNOWN: Warning threshold must be greater than or equal to cricital threshold.\n";
	exit( $STATUS{ 'UNKNOWN'});
}

# Get the swap information
my $cmd;
if( $method eq "swap") {
	$cmd = "/usr/sbin/swap -s";
}
elsif( $method eq "top") {
	$cmd = "/usr/local/bin/top -n | /usr/bin/grep \"^Memory\"";
}
my $cmdout;
if( open( CMD, "$cmd|")) {
	$cmdout = <CMD>;
	unless( close( CMD)) {
		print "UNKNOWN: Unable to terminate command '$cmd'";
		exit( $STATUS{ "UNKNOWN"});
	}
}
else {
	print "UNKNOWN: Unable to execute '$cmd'";
	exit( $STATUS{ "UNKNOWN"});
}

# Validate the swap information
my( $allocated, $reserved, $used, $available, $total) = ( 0, 0, 0, 0, 0);
if( $method eq "swap" && $cmdout =~ /^total: (\d+)k bytes allocated \+ (\d+)k reserved = (\d+)k used, (\d+)k available$/) {
	( $allocated, $reserved, $used, $available) = ( $1, $2, $3, $4);
	$allocated *= 1024;
	$reserved *= 1024;
	$used *= 1024;
	$available *= 1024;
	$total = $available + $used;
}
elsif( $method eq "top" && $cmdout =~ /^Memory: \d+M phys mem, \d+M free mem, (\d+)M total swap, (\d+)M free swap$/) {
	( $total, $available) = ( $1, $2);
	$total *= 1024**2;
	$available *= 1024**2;
}
else {
	print "UNKNOWN: Unknown output from '$cmd': '$cmdout'\n";
	exit( $STATUS{ 'UNKNOWN'});
}

# Check threshold values
my $perfdata = "";
if( $method eq "swap") {
	$perfdata = sprintf( "available=%lukB allocated=%lukB reserved=%lukB used=%lukB",
			$available / 1024, $allocated / 1024, $reserved / 1024, $used / 1024);
}
elsif( $method eq "top") {
	$perfdata = sprintf( "available=%luMB total=%luMB", $available / 1024**2, 
			$total / 1024**2);
}

if( $threshold_type eq "percent") {
	# Convert values to percentages
	$available /= $total;
	$available *= 100;
	$allocated /= $total;
	$allocated *= 100;
	$reserved /= $total;
	$reserved *= 100;
	$used /= $total;
	$used *= 100;
	if( $available <= $critical_level) {
		printf( "CRITICAL: Available swap (%.1f%%) < critical threshold (%.1f%%)",
				$available, $critical_level);
		print "|$perfdata\n";
		exit( $STATUS{ 'CRITICAL'});
	}
	elsif( $available <= $warning_level) {
		printf( "WARNING: Available swap (%.1f%%) < warning threshold (%.1f%%)",
				$available, $warning_level);
		print "|$perfdata\n";
		exit( $STATUS{ 'WARNING'});
	}
	else {
		printf( "OK: Available swap (%.1f%%) > warning threshold (%.1f%%)",
				$available, $warning_level);
		print "|$perfdata\n";
		exit( $STATUS{ 'OK'});
	}
}
else {
	if( $available <= $critical_level) {
		printf( "CRITICAL: Available swap (%s) < critial threshold (%s)",
				bytes2uom( $available, $critical_uom), 
				bytes2uom( $critical_level, $critical_uom));
		print "|$perfdata\n";
		exit( $STATUS{ 'CRITICAL'});
	}
	elsif( $available <= $warning_level) {
		printf( "WARNING: Available swap (%s) < warning threshold (%s)",
				bytes2uom( $available, $warning_uom), 
				bytes2uom( $warning_level, $warning_uom));
		print "|$perfdata\n";
		exit( $STATUS{ 'WARNING'});
	}
	else {
		printf( "OK: Available swap (%s) > warning threshold (%s)",
				bytes2uom( $available, $warning_uom), 
				bytes2uom( $warning_level, $warning_uom));
		print "|$perfdata\n";
		exit( $STATUS{ 'OK'});
	}
}
